@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24154 $
+ * $Revision: 24917 $
  * $Author: rjongbloed $
- * $Date: 2010-03-23 20:56:23 -0500 (Tue, 23 Mar 2010) $
+ * $Date: 2010-12-02 20:26:58 -0600 (Thu, 02 Dec 2010) $
  */
 
 #ifndef OPAL_SIP_SDP_H
@@ -202,6 +202,22 @@ class SDPMediaDescription : public PObject
 
 PARRAY(SDPMediaDescriptionArray, SDPMediaDescription);
 
+
+class SDPDummyMediaDescription : public SDPMediaDescription
+{
+  PCLASSINFO(SDPDummyMediaDescription, SDPMediaDescription);
+  public:
+    SDPDummyMediaDescription(const OpalTransportAddress & address, const PStringArray & tokens);
+    virtual PString GetSDPMediaType() const;
+    virtual PCaselessString GetSDPTransportType() const;
+    virtual SDPMediaFormat * CreateSDPMediaFormat(const PString & portString);
+    virtual PString GetSDPPortList() const;
+
+  private:
+    PStringArray m_tokens;
+};
+
+
 /////////////////////////////////////////////////////////
 //
 //  SDP media description for media classes using RTP/AVP transport (audio and video)
@@ -282,11 +298,11 @@ class SDPSessionDescription : public PObject
     PString Encode() const;
     PBoolean Decode(const PString & str);
 
-    void SetSessionName(const PString & v) { sessionName = v; }
-    PString GetSessionName() const         { return sessionName; }
+    void SetSessionName(const PString & v);
+    PString GetSessionName() const { return sessionName; }
 
-    void SetUserName(const PString & v)    { ownerUsername = v; }
-    PString GetUserName() const            { return ownerUsername; }
+    void SetUserName(const PString & v);
+    PString GetUserName() const { return ownerUsername; }
 
     const SDPMediaDescriptionArray & GetMediaDescriptions() const { return mediaDescriptions; }
 
