@@ -45,6 +45,7 @@
 
 #ifndef PATH_SEP
 #ifdef _WIN32
+#pragma pack(16)
 #define	PATH_SEP  ";"
 #else
 #define	PATH_SEP  ":"
@@ -215,7 +216,7 @@ bool FFMPEGLibrary::Load(int ver)
 
   if (libAvcodec.Open("libavcodec"))
     seperateLibAvutil = false;
-  else if (libAvcodec.Open("avcodec-51") || libAvcodec.Open("avcodec-52"))
+  else if (libAvcodec.Open("avcodec-52") || libAvcodec.Open("avcodec-51"))
     seperateLibAvutil = true;
   else {
     TRACE (1, _codecString << "\tDYNA\tFailed to load FFMPEG libavcodec library");
@@ -394,6 +395,8 @@ bool FFMPEGLibrary::Load(int ver)
 
 AVCodec *FFMPEGLibrary::AvcodecFindEncoder(enum CodecID id)
 {
+  char dummy[16];
+
   WITH_ALIGNED_STACK({
     AVCodec *res = Favcodec_find_encoder(id);
     return res;
@@ -402,6 +405,8 @@ AVCodec *FFMPEGLibrary::AvcodecFindEncoder(enum CodecID id)
 
 AVCodec *FFMPEGLibrary::AvcodecFindDecoder(enum CodecID id)
 {
+  char dummy[16];
+
   WaitAndSignal m(processLock);
 
   WITH_ALIGNED_STACK({
@@ -412,6 +417,8 @@ AVCodec *FFMPEGLibrary::AvcodecFindDecoder(enum CodecID id)
 
 AVCodecContext *FFMPEGLibrary::AvcodecAllocContext(void)
 {
+  char dummy[16];
+
   WaitAndSignal m(processLock);
 
   WITH_ALIGNED_STACK({
@@ -422,6 +429,8 @@ AVCodecContext *FFMPEGLibrary::AvcodecAllocContext(void)
 
 AVFrame *FFMPEGLibrary::AvcodecAllocFrame(void)
 {
+  char dummy[16];
+
   WaitAndSignal m(processLock);
 
   WITH_ALIGNED_STACK({
@@ -432,6 +441,8 @@ AVFrame *FFMPEGLibrary::AvcodecAllocFrame(void)
 
 int FFMPEGLibrary::AvcodecOpen(AVCodecContext *ctx, AVCodec *codec)
 {
+  char dummy[16];
+
   WaitAndSignal m(processLock);
 
   WITH_ALIGNED_STACK({
@@ -441,6 +452,8 @@ int FFMPEGLibrary::AvcodecOpen(AVCodecContext *ctx, AVCodec *codec)
 
 int FFMPEGLibrary::AvcodecClose(AVCodecContext *ctx)
 {
+  char dummy[16];
+
   WaitAndSignal m(processLock);
 
   WITH_ALIGNED_STACK({
@@ -450,6 +463,8 @@ int FFMPEGLibrary::AvcodecClose(AVCodecContext *ctx)
 
 int FFMPEGLibrary::AvcodecEncodeVideo(AVCodecContext *ctx, BYTE *buf, int buf_size, const AVFrame *pict)
 {
+  char dummy[16];
+
   WITH_ALIGNED_STACK({
     int res = Favcodec_encode_video(ctx, buf, buf_size, pict);
 
@@ -460,6 +475,8 @@ int FFMPEGLibrary::AvcodecEncodeVideo(AVCodecContext *ctx, BYTE *buf, int buf_si
 
 int FFMPEGLibrary::AvcodecDecodeVideo(AVCodecContext *ctx, AVFrame *pict, int *got_picture_ptr, BYTE *buf, int buf_size)
 {
+  char dummy[16];
+
   WITH_ALIGNED_STACK({
     int res = Favcodec_decode_video(ctx, pict, got_picture_ptr, buf, buf_size);
 
@@ -470,6 +487,8 @@ int FFMPEGLibrary::AvcodecDecodeVideo(AVCodecContext *ctx, AVFrame *pict, int *g
 
 void FFMPEGLibrary::AvcodecFree(void * ptr)
 {
+  char dummy[16];
+
   WaitAndSignal m(processLock);
 
   WITH_ALIGNED_STACK({
@@ -479,6 +498,8 @@ void FFMPEGLibrary::AvcodecFree(void * ptr)
 
 void FFMPEGLibrary::AvSetDimensions(AVCodecContext *s, int width, int height)
 {
+  char dummy[16];
+
   WaitAndSignal m(processLock);
 
   WITH_ALIGNED_STACK({
@@ -489,6 +510,8 @@ void FFMPEGLibrary::AvSetDimensions(AVCodecContext *s, int width, int height)
 
 void FFMPEGLibrary::AvLogSetLevel(int level)
 {
+  char dummy[16];
+
   WITH_ALIGNED_STACK({
     FAv_log_set_level(level);
   });
@@ -496,6 +519,8 @@ void FFMPEGLibrary::AvLogSetLevel(int level)
 
 void FFMPEGLibrary::AvLogSetCallback(void (*callback)(void*, int, const char*, va_list))
 {
+  char dummy[16];
+
   WITH_ALIGNED_STACK({
     FAv_log_set_callback(callback);
   });
@@ -503,6 +528,8 @@ void FFMPEGLibrary::AvLogSetCallback(void (*callback)(void*, int, const char*, v
 
 int FFMPEGLibrary::FFCheckAlignment(void)
 {
+  char dummy[16];
+
   if (Fff_check_alignment == NULL) {
     TRACE(1, _codecString << "\tDYNA\tff_check_alignment is not supported by libavcodec.so - skipping check");
     return 0;
