@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23930 $
+ * $Revision: 24775 $
  * $Author: rjongbloed $
- * $Date: 2010-01-13 00:50:09 -0600 (Wed, 13 Jan 2010) $
+ * $Date: 2010-10-05 23:57:47 -0500 (Tue, 05 Oct 2010) $
  */
 
 #include "precompile.h"
@@ -219,7 +219,7 @@ void ConfOPAL::Main()
   if (args.HasOption('a')) {
     PFilePath path = args.GetOptionString('a');
     ivr->SetDefaultVXML(path);
-    m_manager->AddRouteEntry(".*:.* = ivr:");
+    m_manager->AddRouteEntry(".*:.* = ivr:<da>");
     cout << "Using attendant IVR " << path << endl;
   }
   else {
