@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23341 $
+ * $Revision: 24363 $
  * $Author: rjongbloed $
- * $Date: 2009-09-05 04:38:32 +0000 (Sat, 05 Sep 2009) $
+ * $Date: 2010-05-21 00:57:20 -0500 (Fri, 21 May 2010) $
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -51,6 +51,8 @@
 #include <h323/h323.h>
 #include <h323/gkclient.h>
 #include <sip/sip.h>
+#include <t38/t38proto.h>
+#include <ptlib/wxstring.h>
 
 
 #include <list>
@@ -72,62 +74,6 @@ class wxGrid;
 class wxConfigBase;
 
 
-class PwxString : public wxString
-{
-  public:
-    PwxString() { }
-    PwxString(const wxString & str) : wxString(str) { }
-    PwxString(const PString & str) : wxString(str, wxConvUTF8 ) { }
-    PwxString(const PFilePath & fn) : wxString(fn, wxConvUTF8 ) { }
-    PwxString(const char * str) : wxString(str, wxConvUTF8) { }
-    PwxString(const OpalMediaFormat & fmt) : wxString(fmt, wxConvUTF8) { }
-#if wxUSE_UNICODE
-    PwxString(const wchar_t * wstr) : wxString(wstr) { }
-#endif
-
-    inline PwxString & operator=(const char * str)     { *this = wxString::wxString(str, wxConvUTF8); return *this; }
-#if wxUSE_UNICODE
-    inline PwxString & operator=(const wchar_t * wstr) { wxString::operator=(wstr); return *this; }
-#endif
-    inline PwxString & operator=(const wxString & str) { wxString::operator=(str); return *this; }
-    inline PwxString & operator=(const PString & str)  { *this = wxString::wxString(str, wxConvUTF8); return *this; }
-
-    inline bool operator==(const char * other)            const { return IsSameAs(wxString(other, wxConvUTF8)); }
-#if wxUSE_UNICODE
-    inline bool operator==(const wchar_t * other)         const { return IsSameAs(other); }
-#endif
-    inline bool operator==(const wxString & other)        const { return IsSameAs(other); }
-    inline bool operator==(const PString & other)         const { return IsSameAs(wxString(other, wxConvUTF8)); }
-    inline bool operator==(const PwxString & other)       const { return IsSameAs(other); }
-    inline bool operator==(const OpalMediaFormat & other) const { return IsSameAs(wxString(other, wxConvUTF8)); }
-
-    inline bool operator!=(const char * other)            const { return !IsSameAs(wxString(other, wxConvUTF8)); }
-#if wxUSE_UNICODE
-    inline bool operator!=(const wchar_t * other)         const { return !IsSameAs(other); }
-#endif
-    inline bool operator!=(const wxString & other)        const { return !IsSameAs(other); }
-    inline bool operator!=(const PString & other)         const { return !IsSameAs(wxString(other, wxConvUTF8)); }
-    inline bool operator!=(const PwxString & other)       const { return !IsSameAs(other); }
-    inline bool operator!=(const OpalMediaFormat & other) const { return !IsSameAs(wxString(other, wxConvUTF8)); }
-
-#if wxUSE_UNICODE
-    inline PString p_str() const { return ToUTF8().data(); }
-    inline operator PString() const { return ToUTF8().data(); }
-    inline operator PFilePath() const { return ToUTF8().data(); }
-    inline operator PIPSocket::Address() const { return PString(ToUTF8().data()); }
-    inline friend ostream & operator<<(ostream & stream, const PwxString & string) { return stream << string.ToUTF8(); }
-    inline friend wostream & operator<<(wostream & stream, const PwxString & string) { return stream << string.c_str(); }
-#else
-    inline PString p_str() const { return c_str(); }
-    inline operator PString() const { return c_str(); }
-    inline operator PFilePath() const { return c_str(); }
-    inline operator PIPSocket::Address() const { return c_str(); }
-    inline friend ostream & operator<<(ostream & stream, const PwxString & string) { return stream << string.c_str(); }
-    inline friend wostream & operator<<(wostream & stream, const PwxString & string) { return stream << string.c_str(); }
-#endif
-};
-
-
 class MyPCSSEndPoint : public OpalPCSSEndPoint
 {
   PCLASSINFO(MyPCSSEndPoint, OpalPCSSEndPoint);
@@ -136,21 +82,34 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
     MyPCSSEndPoint(MyManager & manager);
 
     virtual OpalMediaFormatList GetMediaFormats() const;
+    virtual PSoundChannel * CreateSoundChannel(const OpalPCSSConnection & connection, const OpalMediaFormat & mediaFormat, PBoolean isSource);
+
+    MyManager & m_manager;
+
+    bool TransmitExternalIM(
+      OpalConnection & conn, 
+      const OpalMediaFormat & format, 
+      RTP_IMFrame & frame
+    );
 
   private:
     virtual PBoolean OnShowIncoming(const OpalPCSSConnection & connection);
     virtual PBoolean OnShowOutgoing(const OpalPCSSConnection & connection);
-
-    MyManager & m_manager;
 };
 
 class IMDialog;
 
-struct MessageInfo {
-  PString callId;
-  SIPURL from;
-  PString remoteContact;
-  PString body;
+struct ReceivedMessageInfo {
+  ReceivedMessageInfo(const std::string & connId_, const OpalMediaFormat & fmt_)
+    : m_connId(connId_), m_mediaFormat(fmt_) { }
+
+  std::string m_connId;
+  OpalMediaFormat m_mediaFormat;
+  PURL    m_localURL;
+  PURL    m_remoteURL;
+  PString m_remoteName;
+  PString m_contentType;
+  PString m_body;
 };
 
 struct PresenceInfo {
@@ -195,7 +154,6 @@ class MySIPEndPoint : public SIPEndPoint
     virtual void OnDialogInfoReceived(
       const SIPDialogNotification & info  ///< Information on dialog state change
     );
-    virtual void OnMessageReceived(const SIPURL & from, const SIP_PDU & pdu);
     virtual void OnPresenceInfoReceived(const SIPPresenceInfo & info);
 
     MyManager & m_manager;
@@ -203,6 +161,18 @@ class MySIPEndPoint : public SIPEndPoint
 #endif // OPAL_SIP
 
 
+#if OPAL_FAX
+class MyFaxEndPoint : public OpalFaxEndPoint
+{
+    PCLASSINFO(MyFaxEndPoint, OpalFaxEndPoint)
+
+  public:
+    MyFaxEndPoint(OpalManager & manager) : OpalFaxEndPoint(manager) { }
+    virtual void OnFaxCompleted(OpalFaxConnection & connection, bool timeout);
+};
+#endif // OPAL_FAX
+
+
 class PresenceDialog : public wxDialog
 {
   public:
@@ -249,6 +219,20 @@ class VideoControlDialog : public wxDialog
 };
 
 
+class StartVideoDialog : public wxDialog
+{
+  public:
+    StartVideoDialog(MyManager * manager, bool secondary);
+
+    PwxString m_device;
+    bool      m_preview;
+    int       m_contentRole;
+
+  private:
+    DECLARE_EVENT_TABLE()
+};
+
+
 class CallDialog : public wxDialog
 {
   public:
@@ -290,13 +274,17 @@ class CallIMDialog : public wxDialog
 class IMDialog : public wxDialog
 {
   public:
-    IMDialog(MyManager * manager, const PString & callId, const SIPURL & them, const PString & remoteContact);
+    IMDialog(MyManager * manager, 
+     const std::string & callId,
+ const OpalMediaFormat & m_format,
+         const PString & localName, 
+            const PURL & remoteAddress, 
+         const PString & remoteName);
     ~IMDialog();
 
     void AddTextToScreen(const wxString & text, bool fromUs);
 
-    PString   m_callId;
-    PString   m_remoteContact;
+    std::string m_connId;
 
   private:
     void OnSend(wxCommandEvent & event);
@@ -306,9 +294,12 @@ class IMDialog : public wxDialog
     void SendCurrentText();
 
     MyManager * m_manager;
+    PURL m_remoteAddress;
 
-    PwxString m_them;
-    PwxString m_us;
+    wxString m_them;
+    PString m_us;
+    PString m_remoteAddrStr;
+    OpalMediaFormat m_mediaFormat;
 
     wxTextCtrl * m_enteredText;
     wxTextCtrl * m_textArea;
@@ -465,6 +456,7 @@ class InCallPanel : public CallPanelBase
     void SpeakerVolume(wxScrollEvent & event);
     void MicrophoneVolume(wxScrollEvent & event);
     void SetVolume(bool microphone, int value, bool muted);
+    void UpdateStatistics();
 
     wxButton  * m_Hold;
     wxButton  * m_SpeakerHandset;
@@ -515,12 +507,15 @@ class SpeedDialDialog : public wxDialog
 class RegistrationInfo
 {
   public:
+    // these must match the drop-down box
+    // on the Registration/Subcription dialog box
     enum Types {
       Register,
       SubscribeMWI,
-      SusbcribePresence,
+      SubcribePresence,
       SubscribeMLA,
-      PublishPresence
+      PublishPresence,
+      WatchPresence
     };
 
     RegistrationInfo();
@@ -627,7 +622,7 @@ class OptionsDialog : public wxDialog
     // Sound fields
     PwxString m_SoundPlayer;
     PwxString m_SoundRecorder;
-    int       m_SoundBuffers;
+    int       m_SoundBufferTime;
     PwxString m_LineInterfaceDevice;
     int       m_AEC;
     PwxString m_Country;
@@ -659,22 +654,29 @@ class OptionsDialog : public wxDialog
     PwxString m_VideoMinFrameSize;
     PwxString m_VideoMaxFrameSize;
 
+    wxComboBox * m_videoGrabDevice;
+    wxChoice   * m_videoSourceChoice;
+
+    void AdjustVideoControls(const PwxString & device);
+    void ChangeVideoGrabber(wxCommandEvent & event);
     void TestVideoCapture(wxCommandEvent & event);
 
     ////////////////////////////////////////
     // Fax fields
     PwxString m_FaxStationIdentifier;
     PwxString m_FaxReceiveDirectory;
-    PwxString m_FaxSpanDSP;
-    int       m_FaxSyncMode;
+    int       m_FaxAutoAnswerMode;
     void BrowseFaxDirectory(wxCommandEvent & event);
-    void BrowseFaxSpanDSP(wxCommandEvent & event);
 
     ////////////////////////////////////////
     // IVR fields
 #if OPAL_IVR
     PwxString m_IVRScript;
 #endif
+    int       m_AudioRecordingMode;
+    PwxString m_AudioRecordingFormat;
+    int       m_VideoRecordingMode;
+    PwxString m_VideoRecordingSize;
 
     ////////////////////////////////////////
     // Codec fields
@@ -719,6 +721,8 @@ class OptionsDialog : public wxDialog
     bool         m_DisableFastStart;
     bool         m_DisableH245Tunneling;
     bool         m_DisableH245inSETUP;
+    int          m_ExtendedVideoRoles;
+    bool         m_EnableH239Control;
     wxListBox  * m_Aliases;
     wxTextCtrl * m_NewAlias;
     wxButton   * m_AddAlias;
@@ -757,6 +761,8 @@ class OptionsDialog : public wxDialog
 
     ////////////////////////////////////////
     // Routing fields
+    wxString     m_ForwardingAddress;
+    int          m_ForwardingTimeout;
     wxListCtrl * m_Routes;
     int          m_SelectedRoute;
     wxComboBox * m_RouteSource;
@@ -803,14 +809,10 @@ class MyMedia
 {
 public:
   MyMedia();
-  MyMedia(
-    const char * source,
-    const PString & format
-  );
+  MyMedia(const OpalMediaFormat & format);
 
   bool operator<(const MyMedia & other) { return preferenceOrder < other.preferenceOrder; }
 
-  PString         sourceProtocol;
   OpalMediaFormat mediaFormat;
   const wxChar  * validProtocols;
   int             preferenceOrder;
@@ -835,20 +837,30 @@ class MyManager : public wxFrame, public OpalManager
     void RejectCall();
     void HangUpCall();
     void SendUserInput(char tone);
-    void OnRinging(const OpalPCSSConnection & connection);
 
     PSafePtr<OpalCall>       GetCall(PSafetyMode mode);
     PSafePtr<OpalConnection> GetConnection(bool user, PSafetyMode mode);
 
     bool HasHandset() const;
 
-    enum AnswerModes {
+    enum FaxAnswerModes {
       AnswerVoice,
       AnswerFax,
       AnswerDetect
-    } m_AnswerMode;
+    };
+    FaxAnswerModes m_currentAnswerMode;
+    FaxAnswerModes m_defaultAnswerMode;
     void SwitchToFax();
 
+    MyPCSSEndPoint & GetPCSSEP() { return *pcssEP; }
+
+    void PostEvent(
+      const wxEventType & type,
+      unsigned id,
+      const PString & str = PString::Empty(),
+      const void * data = NULL
+    );
+
   private:
     // OpalManager overrides
     virtual PBoolean OnIncomingConnection(
@@ -889,6 +901,12 @@ class MyManager : public wxFrame, public OpalManager
       unsigned lastDigitTimeout = 4,      ///<  Timeout on last digit in string
       unsigned firstDigitTimeout = 30     ///<  Timeout on receiving any digits
     );
+    virtual PBoolean CreateVideoInputDevice(
+      const OpalConnection & connection,    ///<  Connection needing created video device
+      const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
+      PVideoInputDevice * & device,         ///<  Created device
+      PBoolean & autoDelete                     ///<  Flag for auto delete device
+    );
     virtual PBoolean CreateVideoOutputDevice(
       const OpalConnection & connection,    ///<  Connection needing created video device
       const OpalMediaFormat & mediaFormat,  ///<  Media format for stream
@@ -915,6 +933,7 @@ class MyManager : public wxFrame, public OpalManager
     void OnCallSpeedDialHandset(wxCommandEvent& event);
     void OnSendFax(wxCommandEvent& event);
     void OnSendFaxSpeedDial(wxCommandEvent& event);
+    void OnSendIMSpeedDial(wxCommandEvent& event);
     void OnStartIM(wxCommandEvent& event);
     void OnMenuAnswer(wxCommandEvent& event);
     void OnMenuHangUp(wxCommandEvent& event);
@@ -934,6 +953,7 @@ class MyManager : public wxFrame, public OpalManager
     void OnTransfer(wxCommandEvent& event);
     void OnStartRecording(wxCommandEvent& event);
     void OnStopRecording(wxCommandEvent& event);
+    void OnSendAudioFile(wxCommandEvent& event);
     void OnAudioDevicePair(wxCommandEvent& event);
     void OnAudioDevicePreset(wxCommandEvent& event);
     void OnNewCodec(wxCommandEvent& event);
@@ -941,14 +961,22 @@ class MyManager : public wxFrame, public OpalManager
     void OnStopVideo(wxCommandEvent& event);
     void OnVFU(wxCommandEvent& event);
     void OnVideoControl(wxCommandEvent& event);
+    void OnDefVidWinPos(wxCommandEvent& event);
     void OnSashPositioned(wxSplitterEvent& event);
     void OnSpeedDialActivated(wxListEvent& event);
     void OnSpeedDialColumnResize(wxListEvent& event);
     void OnRightClick(wxListEvent& event);
 
     void OnMyPresence(wxCommandEvent& event);
+    void OnStartInstantMessage(wxCommandEvent& event);
+
+    void OnEvtRinging(wxCommandEvent & theEvent);
+    void OnEvtEstablished(wxCommandEvent & theEvent);
+    void OnEvtOnHold(wxCommandEvent & theEvent);
+    void OnEvtCleared(wxCommandEvent & theEvent);
 
     bool CanDoFax() const;
+    bool CanDoIM() const;
 
     enum SpeedDialViews {
       e_ViewLarge,
@@ -1023,13 +1051,12 @@ class MyManager : public wxFrame, public OpalManager
     typedef std::map<std::string, IMDialog *> ConversationMapType;
     PMutex conversationMapMutex;
 
-    void OnMessageReceived(MessageInfo * info);
-    IMDialog * GetOrCreateConversation(const MessageInfo & messageInfo);
+    IMDialog * GetOrCreateConversation(const ReceivedMessageInfo & messageInfo);
 
     ConversationMapType conversationMap;
 
+    bool SubscribePresence(wxString & uri);
     void OnPresenceInfoReceived(const SIPPresenceInfo & info);
-
 #endif
 
 #if OPAL_IVR
@@ -1044,18 +1071,35 @@ class MyManager : public wxFrame, public OpalManager
     PwxString m_LastDialed;
     PwxString m_LastReceived;
 
+    PwxString m_ForwardingAddress;
+    int       m_ForwardingTimeout;
+    PTimer    m_ForwardingTimer;
+    PDECLARE_NOTIFIER(PTimer, MyManager, OnForwardingTimeout);
+
     bool m_VideoGrabPreview;
     int  m_localVideoFrameX;
     int  m_localVideoFrameY;
     int  m_remoteVideoFrameX;
     int  m_remoteVideoFrameY;
+
+    PVideoDevice::OpenArgs m_SecondaryVideoGrabber;
+    bool                   m_SecondaryVideoGrabPreview;
+    bool                   m_SecondaryVideoOpening;
+
     PwxString m_VideoGrabFrameSize;
     PwxString m_VideoMinFrameSize;
     PwxString m_VideoMaxFrameSize;
-    bool AdjustFrameSize();
+    bool AdjustVideoFormats();
+
+    enum ExtendedVideoRoles {
+      e_DisabledExtendedVideoRoles,
+      e_ExtendedVideoRolePerOption,
+      e_ForcePresentationVideoRole,
+      e_ForceLiveVideoRole
+    } m_ExtendedVideoRoles;
 
     MyMediaList m_mediaInfo;
-    void InitMediaInfo(const char * source, const OpalMediaFormatList & formats);
+    void InitMediaInfo(const OpalMediaFormatList & formats);
     void ApplyMediaInfo();
 
 #if PTRACING
@@ -1071,20 +1115,6 @@ class MyManager : public wxFrame, public OpalManager
     void StopRingSound();
     void UpdateAudioDevices();
 
-    enum CallState {
-      IdleState,
-      CallingState,
-      RingingState,
-      AnsweringState,
-      InCallState,
-      ClearingCallState
-    } m_callState;
-    friend ostream & operator<<(ostream & strm, CallState state);
-    void SetState(
-      CallState newState,
-      const PString & token
-    );
-
     PString            m_incomingToken;
     PSafePtr<OpalCall> m_activeCall;
 
@@ -1109,7 +1139,9 @@ class MyManager : public wxFrame, public OpalManager
     };
     list<CallsOnHold>    m_callsOnHold;
 
-    PwxString m_lastRecordFile;
+    OpalRecordManager::Options m_recordingOptions;
+    PwxString                  m_lastRecordFile;
+    PwxString                  m_lastPlayFile;
 
     DECLARE_EVENT_TABLE()
 
