@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22141 $
- * $Author: rjongbloed $
- * $Date: 2009-03-02 20:26:18 -0600 (Mon, 02 Mar 2009) $
+ * $Revision: 23525 $
+ * $Author: csoutheren $
+ * $Date: 2009-09-24 01:02:24 -0500 (Thu, 24 Sep 2009) $
  */
 
 #include <ptlib.h>
@@ -59,7 +59,6 @@
 
 #define new PNEW
 
-
 PCREATE_PROCESS(SimpleOpalProcess);
 
 ///////////////////////////////////////////////////////////////
@@ -148,6 +147,7 @@ void SimpleOpalProcess::Main()
              "-udp-max:"
              "-use-long-mime."
 #if OPAL_VIDEO
+             "C-ratecontrol:"
              "-rx-video." "-no-rx-video."
              "-tx-video." "-no-tx-video."
              "-grabber:"
@@ -157,6 +157,7 @@ void SimpleOpalProcess::Main()
              "-displaydriver:"
              "-video-size:"
              "-video-rate:"
+             "-video-bitrate:"
 #endif
 #if OPAL_IVR
              "V-no-ivr."
@@ -206,6 +207,8 @@ void SimpleOpalProcess::Main()
             "     --video-size size    : Set the size of the video for all video formats, use\n"
             "                          : \"qcif\", \"cif\", WxH etc\n"
             "     --video-rate rate    : Set the frame rate of video for all video formats\n"
+            "     --video-bitrate rate : Set the bit rate for all video formats\n"
+            "     -C string            : Enable and select video rate control algorithm\n"
             "\n"
 #endif
 
@@ -423,6 +426,8 @@ PBoolean MyManager::Initialise(PArgList & args)
     video.deviceName = args.GetOptionString("grabber");
     video.driverName = args.GetOptionString("grabdriver");
     video.channelNumber = args.GetOptionString("grabchannel").AsInteger();
+    if (args.HasOption("video-rate")) 
+      video.rate = args.GetOptionString("video-rate").AsUnsigned();
     if (!SetVideoInputDevice(video)) {
       cerr << "Unknown grabber device " << video.deviceName << "\n"
               "Available devices are:" << setfill(',') << PVideoInputDevice::GetDriversDeviceNames("") << endl;
@@ -562,8 +567,6 @@ PBoolean MyManager::Initialise(PArgList & args)
   if (!args.HasOption('S')) {
     pcssEP = new MyPCSSEndPoint(*this);
 
-    pcssEP->SetSoundChannelBufferDepth(10);
-
     pcssEP->autoAnswer = args.HasOption('a');
     cout << "Auto answer is " << (pcssEP->autoAnswer ? "on" : "off") << "\n";
           
@@ -665,13 +668,16 @@ PBoolean MyManager::Initialise(PArgList & args)
     cout <<  "SIP started on " << setfill(',') << sipEP->GetListeners() << setfill(' ') << endl;
 
     if (args.HasOption('r')) {
-      PString registrar = args.GetOptionString('r');
-      cout << "Using SIP registrar " << registrar << " ... " << flush;
-      if (sipEP->Register(registrar, args.GetOptionString('u'), args.GetOptionString('u'), args.GetOptionString('p'), args.GetOptionString("sip-domain")))
-        cout << "done.";
+      SIPRegister::Params params;
+      params.m_registrarAddress = args.GetOptionString('r');
+      params.m_addressOfRecord = args.GetOptionString('u');
+      params.m_password = args.GetOptionString('p');
+      params.m_realm = args.GetOptionString("sip-domain");
+      PString aor;
+      if (sipEP->Register(params, aor))
+        cout << "Using SIP registrar " << params.m_registrarAddress << " for " << aor << endl;
       else
-        cout << "failed!";
-      cout << endl;
+        cout << "Could not use SIP registrar " << params.m_registrarAddress << endl;
       pauseBeforeDialing = PTrue;
     }
   }
@@ -830,6 +836,7 @@ PBoolean MyManager::Initialise(PArgList & args)
           "Available codecs: " << allMediaFormats << setfill(' ') << endl;
 
 #if OPAL_VIDEO
+  PString rcOption = args.GetOptionString('C');
   OpalMediaFormat::GetAllRegisteredMediaFormats(allMediaFormats);
   for (PINDEX i = 0; i < allMediaFormats.GetSize(); i++) {
     OpalMediaFormat mediaFormat = allMediaFormats[i];
@@ -849,8 +856,13 @@ PBoolean MyManager::Initialise(PArgList & args)
         unsigned rate = args.GetOptionString("video-rate").AsUnsigned();
         unsigned frameTime = 90000 / rate;
         mediaFormat.SetOptionInteger(OpalMediaFormat::FrameTimeOption(), frameTime);
-        mediaFormat.SetOptionBoolean("Rate Control Enable", true);
       }
+      if (args.HasOption("video-bitrate")) {
+        unsigned rate = args.GetOptionString("video-bitrate").AsUnsigned();
+        mediaFormat.SetOptionInteger(OpalMediaFormat::TargetBitRateOption(), rate);
+      }
+      if (!rcOption.IsEmpty())
+        mediaFormat.SetOptionString(OpalVideoFormat::RateControllerOption(), rcOption);
       OpalMediaFormat::SetRegisteredMediaFormat(mediaFormat);
     }
   }
@@ -1098,6 +1110,7 @@ void MyManager::Main(PArgList & args)
         cout << help ;
         break;
 
+#if OPAL_HAS_MIXER
       case 'z':
         if (currentCallToken.IsEmpty())
          cout << "Cannot stop or start record whilst no call in progress.\n";
@@ -1110,6 +1123,7 @@ void MyManager::Main(PArgList & args)
           cout << "Recording stopped.\n";
         }
         break;
+#endif
         
       case 'y' :
         if ( pcssEP != NULL &&
@@ -1243,10 +1257,14 @@ void MyManager::TransferCall(const PString & dest)
   }
 
   for (PSafePtr<OpalConnection> connection = call->GetConnection(0); connection != NULL; ++connection) {
-    if (!PIsDescendant(&(*connection), OpalPCSSConnection) && !PIsDescendant(&(*connection), OpalLineConnection)) {
-      connection->TransferConnection(dest.IsEmpty() ? heldCallToken : dest);
+    if (PIsDescendant(&(*connection), OpalPCSSConnection))
       break;
-    }
+#if OPAL_LID
+    if (PIsDescendant(&(*connection), OpalLineConnection))
+      break;
+#endif
+    connection->TransferConnection(dest.IsEmpty() ? heldCallToken : dest);
+    break;
   }
 }
 
