@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21964 $
+ * $Revision: 24048 $
  * $Author: rjongbloed $
- * $Date: 2009-01-28 20:07:30 -0600 (Wed, 28 Jan 2009) $
+ * $Date: 2010-02-10 17:45:14 -0600 (Wed, 10 Feb 2010) $
  */
 
 #include <ptlib.h>
@@ -138,9 +138,9 @@ class H323_iLBCCapability : public H323GenericAudioCapability
 #endif // OPAL_H323
 
 
-const OpalMediaFormat & GetOpaliLBC()
+const OpalAudioFormat & GetOpaliLBC()
 {
-  static OpalMediaFormat const iLBC_Format(new OpaliLBCFormat);
+  static OpalAudioFormat const iLBC_Format(new OpaliLBCFormat);
 
 #if OPAL_H323
   static H323CapabilityFactory::Worker<H323_iLBCCapability> iLBC_Factory(OPAL_iLBC, true);
