@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21028 $
- * $Author: csoutheren $
- * $Date: 2008-09-16 18:32:30 -0500 (Tue, 16 Sep 2008) $
+ * $Revision: 22444 $
+ * $Author: rjongbloed $
+ * $Date: 2009-04-20 18:49:06 -0500 (Mon, 20 Apr 2009) $
  */
 
 #ifdef __GNUC__
@@ -41,6 +41,8 @@
 #include <ptclib/dtmf.h>
 
 
+PFACTORY_CREATE(PFactory<PPluginModuleManager>, OpalPluginLIDManager, "OpalPluginLIDManager", true);
+
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalPluginLIDManager::OpalPluginLIDManager(PPluginManager * mgr)
