@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22556 $
+ * $Revision: 24212 $
  * $Author: rjongbloed $
- * $Date: 2009-05-07 09:02:51 +0000 (Thu, 07 May 2009) $
+ * $Date: 2010-04-07 22:03:34 -0500 (Wed, 07 Apr 2010) $
  */
 
 #include <ptlib.h>
@@ -47,8 +47,9 @@
 
 OpalLocalEndPoint::OpalLocalEndPoint(OpalManager & mgr, const char * prefix)
   : OpalEndPoint(mgr, prefix, CanTerminateCall)
+  , m_deferredAlerting(false)
 {
-  PTRACE(3, "LocalEP\tCreated endpoint.\n");
+  PTRACE(3, "LocalEP\tCreated endpoint.");
 }
 
 
@@ -58,19 +59,28 @@ OpalLocalEndPoint::~OpalLocalEndPoint()
 }
 
 
-PBoolean OpalLocalEndPoint::MakeConnection(OpalCall & call,
-                                      const PString & /*remoteParty*/,
-                                               void * userData,
-                                       unsigned int   /*options*/,
-                      OpalConnection::StringOptions * /* stringOptions*/)
+OpalMediaFormatList OpalLocalEndPoint::GetMediaFormats() const
 {
-  return AddConnection(CreateConnection(call, userData));
+  return manager.GetCommonMediaFormats(false, true);
 }
 
 
-OpalLocalConnection * OpalLocalEndPoint::CreateConnection(OpalCall & call, void * userData)
+PSafePtr<OpalConnection> OpalLocalEndPoint::MakeConnection(OpalCall & call,
+                                                      const PString & /*remoteParty*/,
+                                                               void * userData,
+                                                         unsigned int options,
+                                      OpalConnection::StringOptions * stringOptions)
 {
-  return new OpalLocalConnection(call, *this, userData);
+  return AddConnection(CreateConnection(call, userData, options, stringOptions));
+}
+
+
+OpalLocalConnection * OpalLocalEndPoint::CreateConnection(OpalCall & call, 
+                                                              void * userData,
+                                                            unsigned options,
+                                     OpalConnection::StringOptions * stringOptions)
+{
+  return new OpalLocalConnection(call, *this, userData, options, stringOptions);
 }
 
 
@@ -87,6 +97,19 @@ bool OpalLocalEndPoint::OnIncomingCall(OpalLocalConnection & connection)
 }
 
 
+bool OpalLocalEndPoint::AlertingIncomingCall(const PString & token)
+{
+  PSafePtr<OpalLocalConnection> connection = GetLocalConnectionWithLock(token, PSafeReadOnly);
+  if (connection == NULL) {
+    PTRACE(2, "LocalEP\tCould not find connection using token \"" << token << '"');
+    return false;
+  }
+
+  connection->AlertingIncoming();
+  return true;
+}
+
+
 bool OpalLocalEndPoint::AcceptIncomingCall(const PString & token)
 {
   PSafePtr<OpalLocalConnection> connection = GetLocalConnectionWithLock(token, PSafeReadOnly);
@@ -164,12 +187,19 @@ bool OpalLocalEndPoint::IsSynchronous() const
 
 /////////////////////////////////////////////////////////////////////////////
 
-static unsigned LastConnectionTokenID;
-
-OpalLocalConnection::OpalLocalConnection(OpalCall & call, OpalLocalEndPoint & ep, void * /*userData*/)
-  : OpalConnection(call, ep, psprintf("%u", ++LastConnectionTokenID))
-  , endpoint(ep), userData(NULL)
+OpalLocalConnection::OpalLocalConnection(OpalCall & call,
+                                OpalLocalEndPoint & ep,
+                                             void * /*userData*/,
+                                           unsigned options,
+                    OpalConnection::StringOptions * stringOptions,
+                                               char tokenPrefix)
+  : OpalConnection(call, ep, ep.GetManager().GetNextToken(tokenPrefix), options, stringOptions)
+  , endpoint(ep)
+  , userData(NULL)
 {
+#if OPAL_PTLIB_DTMF
+  m_sendInBandDTMF = m_detectInBandDTMF = false;
+#endif
   PTRACE(4, "LocalCon\tCreated connection with token \"" << callToken << '"');
 }
 
@@ -197,15 +227,26 @@ PBoolean OpalLocalConnection::SetUpConnection()
       Release(EndedByNoAccept);
       return false;
     }
+  }
+  else if (ownerCall.IsEstablished()) {
+    PTRACE(3, "LocalCon\tTransfer of connection in call " << ownerCall);
+    OnApplyStringOptions();
+    AutoStartMediaStreams();
+    OnConnectedInternal();
+  }
+  else {
+    PTRACE(3, "LocalCon\tIncoming call from " << remotePartyName);
 
-    return true;
+    OnApplyStringOptions();
+
+    if (!endpoint.OnIncomingCall(*this))
+      return false;
+
+    if (!endpoint.IsDeferredAlerting())
+      AlertingIncoming();
   }
 
-  PTRACE(3, "LocalCon\tSetUpConnection(" << remotePartyName << ')');
-  SetPhase(AlertingPhase);
-  OnAlerting();
-
-  return endpoint.OnIncomingCall(*this);
+  return true;
 }
 
 
@@ -218,6 +259,15 @@ PBoolean OpalLocalConnection::SetAlerting(const PString & calleeName, PBoolean)
 }
 
 
+PBoolean OpalLocalConnection::SetConnected()
+{
+  PTRACE(3, "LocalCon\tSetConnected()");
+
+  AutoStartMediaStreams(); // if no media streams, try and start them
+
+  return OpalConnection::SetConnected();
+}
+
 OpalMediaStream * OpalLocalConnection::CreateMediaStream(const OpalMediaFormat & mediaFormat,
                                                          unsigned sessionID,
                                                          PBoolean isSource)
@@ -226,6 +276,22 @@ OpalMediaStream * OpalLocalConnection::CreateMediaStream(const OpalMediaFormat &
 }
 
 
+OpalMediaStreamPtr OpalLocalConnection::OpenMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, bool isSource)
+{
+#if OPAL_VIDEO
+  if ( isSource &&
+       mediaFormat.GetMediaType() == OpalMediaType::Video() &&
+      !ownerCall.IsEstablished() &&
+      !endpoint.GetManager().CanAutoStartTransmitVideo()) {
+    PTRACE(3, "LocalCon\tOpenMediaStream auto start disabled, refusing video open");
+    return NULL;
+  }
+#endif
+
+  return OpalConnection::OpenMediaStream(mediaFormat, sessionID, isSource);
+}
+
+
 bool OpalLocalConnection::SendUserInputString(const PString & value)
 {
   PTRACE(3, "LocalCon\tSendUserInputString(" << value << ')');
@@ -233,23 +299,24 @@ bool OpalLocalConnection::SendUserInputString(const PString & value)
 }
 
 
+void OpalLocalConnection::AlertingIncoming()
+{
+  if (LockReadWrite()) {
+    if (GetPhase() < AlertingPhase) {
+      SetPhase(AlertingPhase);
+      OnAlerting();
+    }
+    UnlockReadWrite();
+  }
+}
+
+
 void OpalLocalConnection::AcceptIncoming()
 {
   if (LockReadWrite()) {
+    AlertingIncoming();
+    AutoStartMediaStreams();
     OnConnectedInternal();
-    OpalMediaTypeFactory::KeyList_T mediaTypes = OpalMediaType::GetList();
-    for (OpalMediaTypeFactory::KeyList_T::iterator iter = mediaTypes.begin(); iter != mediaTypes.end(); ++iter) {
-      OpalMediaType mediaType = *iter;
-      switch (GetAutoStart(mediaType)) {
-        case OpalMediaType::Transmit :
-        case OpalMediaType::TransmitReceive :
-          if (GetMediaStream(mediaType, true) == NULL)
-            ownerCall.OpenSourceMediaStreams(*this, mediaType);
-
-        default :
-          break;
-      }
-    }
     UnlockReadWrite();
   }
 }
